import type { RequestMethod, TRequestMethodHasBody } from '../server/handler-types';
import type { AddOptionalsToUndefined, EmptyRecordToNever, MakeObjectWithOptionalKeysUndefinable, ObjectToNever, RemoveNever } from '../server/type-helpers';
export type UrlOverrides = {
    overrideOrigin?: string;
    serverSideOrigin?: string;
};
export type RouteDefinitions = {
    [route: string]: {
        params: Record<string, string | string[]>;
        api: {
            typeSafe: boolean;
            method: RequestMethod;
            body: object;
            routeParams: object;
            queryParams: object;
            cookies: object;
            data: object;
        };
    };
};
export type RequestConfig<RouteData extends RouteDefinitions[keyof RouteDefinitions], Method extends RequestMethod> = MakeObjectWithOptionalKeysUndefinable<AddOptionalsToUndefined<RemoveNever<{
    params: EmptyRecordToNever<RouteData['params']>;
    query: RouteData['api']['typeSafe'] extends true ? ObjectToNever<RouteData['api']['queryParams']> : Record<string, string | string[]> | undefined;
    body: TRequestMethodHasBody[Method] extends true ? RouteData['api']['typeSafe'] extends true ? ObjectToNever<RouteData['api']['body']> : object | string | number | boolean | undefined : never;
}>>>;
export declare function makeApiRequestFunction<Routes extends RouteDefinitions, Method extends RequestMethod>(method: Method, { overrideOrigin, serverSideOrigin }?: UrlOverrides): <Route extends keyof Routes>(route: Route, data: MakeObjectWithOptionalKeysUndefinable<AddOptionalsToUndefined<RemoveNever<{
    params: EmptyRecordToNever<Routes[Route]["params"]>;
    query: Routes[Route]["api"]["typeSafe"] extends true ? ObjectToNever<Routes[Route]["api"]["queryParams"]> : Record<string, string | string[]> | undefined;
    body: {
        GET: false;
        POST: true;
        PUT: true;
        PATCH: true;
        DELETE: false;
        HEAD: false;
        OPTIONS: false;
    }[Method] extends true ? Routes[Route]["api"]["typeSafe"] extends true ? ObjectToNever<Routes[Route]["api"]["body"]> : string | number | boolean | object | undefined : never;
}>>>, options?: RequestInit & {
    overrideOrigin?: string;
}) => Promise<Routes[Route]["api"]["data"]>;
export type RequestErrorType = 'unauthorized' | 'bad-request' | 'not-found' | 'server-error' | 'forbidden' | 'network-error' | 'unknown';
export declare class RequestError extends Error {
    readonly type: RequestErrorType;
    readonly data?: any | null | {
        status: 'error';
        message: string;
    };
    constructor(type: RequestErrorType, message: string, data?: any | null);
    toString(): string;
}
export declare function buildUrl(path: string, params: Record<string, string | string[]>, { overrideOrigin, serverSideOrigin }?: UrlOverrides): string;
export declare function makeBuildUrlFunction<Routes extends RouteDefinitions>(urlOverrides?: UrlOverrides): <Route extends keyof Routes>(route: Route, variables: Routes[Route]["params"]) => string;
