"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "codegen", {
    enumerable: true,
    get: function() {
        return codegen;
    }
});
var _handlertypes = require("../server/handler-types");
function _array_like_to_array(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _array_with_holes(arr) {
    if (Array.isArray(arr)) return arr;
}
function _iterable_to_array_limit(arr, i) {
    var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];
    if (_i == null) return;
    var _arr = [];
    var _n = true;
    var _d = false;
    var _s, _e;
    try {
        for(_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true){
            _arr.push(_s.value);
            if (i && _arr.length === i) break;
        }
    } catch (err) {
        _d = true;
        _e = err;
    } finally{
        try {
            if (!_n && _i["return"] != null) _i["return"]();
        } finally{
            if (_d) throw _e;
        }
    }
    return _arr;
}
function _non_iterable_rest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _sliced_to_array(arr, i) {
    return _array_with_holes(arr) || _iterable_to_array_limit(arr, i) || _unsupported_iterable_to_array(arr, i) || _non_iterable_rest();
}
function _unsupported_iterable_to_array(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _array_like_to_array(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _array_like_to_array(o, minLen);
}
function validateUrl(url, name) {
    if (!url) return;
    try {
        new URL(url);
    } catch (e) {
        throw new Error("".concat(name, " must be a valid origin and include a protocol and host. For example: https://example.com. Currently it is: ").concat(url));
    }
}
function codegen(paths, baseFolderPath, param) {
    var reactQuery = param.reactQuery, overrideOrigin = param.overrideOrigin, serverSideOrigin = param.serverSideOrigin;
    validateUrl(overrideOrigin, "overrideOrigin");
    validateUrl(serverSideOrigin, "serverSideOrigin");
    var imports = [];
    var params = [];
    var asserts = [];
    var routes = {
        DELETE: [],
        GET: [],
        HEAD: [],
        OPTIONS: [],
        PATCH: [],
        POST: [],
        PUT: []
    };
    var currentIndex = 0;
    var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
    try {
        var _loop = function() {
            var path = _step.value;
            var importId = "Route".concat(currentIndex++);
            var importPath = "./" + path.filepath.slice(baseFolderPath.length + 1, -3);
            var methods = Object.entries(path.methods).filter(function(param) {
                var _param = _sliced_to_array(param, 2), hasMethod = _param[1];
                return hasMethod;
            }).map(function(param) {
                var _param = _sliced_to_array(param, 1), method = _param[0];
                return method;
            });
            imports.push("import type { ".concat(methods.map(function(m) {
                return "".concat(m, " as ").concat(importId).concat(m);
            }).join(", "), ' } from "').concat(importPath, '";'));
            var paramsTypeArgs = Object.entries(path.params).map(function(param) {
                var _param = _sliced_to_array(param, 2), _$param = _param[0], type = _param[1];
                return "".concat(_$param, ":").concat(type === "string" ? "string" : "string[]", ";");
            }).join("");
            if (paramsTypeArgs.length > 0) params.push("type ".concat(importId, "Params = {").concat(paramsTypeArgs, "};"));
            else params.push("type ".concat(importId, "Params = Record<string, never>;"));
            methods.forEach(function(m) {
                asserts.push("Assert<APIType<typeof ".concat(importId).concat(m, '>["method"], "').concat(m, '">'));
                asserts.push("Assert<APIType<typeof ".concat(importId).concat(m, '>["routeParams"], ').concat(importId, "Params>"));
                routes[m].push("'".concat(path.url, "': { api: APIType<typeof ").concat(importId).concat(m, ">; params: ").concat(importId, "Params; };"));
            });
        };
        for(var _iterator = paths[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true)_loop();
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally{
        try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
                _iterator.return();
            }
        } finally{
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }
    var routeDict = Object.entries(routes).filter(function(param) {
        var _param = _sliced_to_array(param, 2), r = _param[1];
        return r.length;
    }).map(function(param) {
        var _param = _sliced_to_array(param, 2), method = _param[0], routes = _param[1];
        return "type Routes".concat(method, " = {\n    ").concat(routes.join("\n    "), "\n};");
    });
    var functions = Object.entries(routes).filter(function(param) {
        var _param = _sliced_to_array(param, 2), r = _param[1];
        return r.length;
    }).map(function(param) {
        var _param = _sliced_to_array(param, 1), method = _param[0];
        return "export const api".concat(upperFirst(method), " = makeApiRequestFunction<Routes").concat(method, ", '").concat(method, "'>('").concat(method, "', URL_CONFIG);");
    });
    var reactQueryFunctions = reactQuery ? Object.entries(routes).filter(function(param) {
        var _param = _sliced_to_array(param, 2), r = _param[1];
        return r.length;
    }).map(function(param) {
        var _param = _sliced_to_array(param, 1), method = _param[0];
        var mutation = "export const useApi".concat(upperFirst(method), "Mutation = makeUseApiMutation<Routes").concat(method, ", '").concat(method, "'>('").concat(method, "', useMutation, URL_CONFIG);");
        if (_handlertypes.IS_MUTATION[method]) {
            return mutation;
        } else {
            var query = "export const useApi".concat(upperFirst(method), "Query = makeUseApiQuery<Routes").concat(method, ", '").concat(method, "'>('").concat(method, "', useQuery, URL_CONFIG);");
            var getKeyFn = "export const getApi".concat(upperFirst(method), "QueryKey = makeBuildUrlFunction<Routes").concat(method, ">(URL_CONFIG);");
            return [
                query,
                mutation,
                getKeyFn
            ];
        }
    }).flat() : [];
    var typeHelpers = Object.entries(routes).filter(function(param) {
        var _param = _sliced_to_array(param, 2), r = _param[1];
        return r.length;
    }).map(function(param) {
        var _param = _sliced_to_array(param, 1), method = _param[0];
        return [
            "export type TypeOf".concat(upperFirst(method), "<T extends keyof Routes").concat(method, "> = Routes").concat(method, "[T]['api']['data'];"),
            "export type BodyOf".concat(upperFirst(method), "<T extends keyof Routes").concat(method, "> = Routes").concat(method, "[T]['api']['body'];"),
            "export type ParamsOf".concat(upperFirst(method), "<T extends keyof Routes").concat(method, "> = Routes").concat(method, "[T]['params'];"),
            "export type QueryOf".concat(upperFirst(method), "<T extends keyof Routes").concat(method, "> = Routes").concat(method, "[T]['api']['queryParams'];"),
            "export type ".concat(upperFirst(method), "Route = keyof Routes").concat(method, ";")
        ].join("\n");
    });
    var buildUrlFunctions = Object.entries(routes).filter(function(param) {
        var _param = _sliced_to_array(param, 2), r = _param[1];
        return r.length;
    }).map(function(param) {
        var _param = _sliced_to_array(param, 1), method = _param[0];
        return "export const build".concat(upperFirst(method), "Url = makeBuildUrlFunction<Routes").concat(method, ">(URL_CONFIG);");
    });
    return '/* eslint-disable */\n  \n// Do not edit this file, it is auto-generated. All changes will be lost.\n\nimport type { APIType } from "next-typed-api/client"\nimport { makeApiRequestFunction, makeUseApiMutation, makeUseApiQuery, makeBuildUrlFunction } from "next-typed-api/client"\n'.concat(reactQuery ? "import { useMutation, useQuery } from '@tanstack/react-query';" : "", "  \n\n").concat(imports.join("\n"), "\n\nconst URL_CONFIG = {\n  overrideOrigin: ").concat(overrideOrigin ? "'".concat(overrideOrigin, "'") : "undefined", ",\n  serverSideOrigin: ").concat(serverSideOrigin ? "'".concat(serverSideOrigin, "'") : "undefined", ",\n};\n\n").concat(params.join("\n"), "\n\n").concat(routeDict.join("\n"), "\n\n").concat(functions.join("\n"), "\n\n").concat(buildUrlFunctions.join("\n"), "\n\n").concat(reactQueryFunctions.join("\n"), "\n\n").concat(typeHelpers.join("\n"), "\n\ntype _Asserts = [\n    ").concat(asserts.join(",\n    "), "\n];\n\ntype Assert<T, U extends T> = T;\n");
}
function upperFirst(str) {
    if (str.length === 0) return str;
    return str[0].toUpperCase() + str.slice(1).toLowerCase();
}
